//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#ifndef KMFMENUPAGE_H
#define KMFMENUPAGE_H

#include "kmfwidget.h"
#include <kmediafactory/plugin.h>
#include <qobject.h>
#include <qsize.h>
#include <qptrlist.h>
#include <qdom.h>
#include <Magick++.h>

class KMFButton;
class TemplateObject;
class KMFMenuPage;

typedef QPtrListIterator<KMFMenuPage> KMFMenuPageIterator;

/**
*/
class KMFMenuPage : public KMFWidget
{
    Q_OBJECT
  public:
    KMFMenuPage(QObject *parent = 0, const char *name = 0);
    ~KMFMenuPage();

    Magick::Image* getLayer(Layer layer);
    void addButton(KMFButton* button) { m_buttons->append(button); };
    int buttonCount() { return m_buttons->count(); };
    KMFButton* button(int n) { return m_buttons->at(n); };
    KMFButton* button(const QString& name);
    void writeDvdAuthorXml(QDomElement& element);

    void setResolution(QSize resolution);
    const QSize& resolution() { return m_resolution; };
    bool makeMpeg(QString projectPath);
    Magick::Image& layer(Layer layer);
    bool paint();
    void fromXML(const QDomElement& element);
    bool parseButtons(bool addPages = true);

    int titleStart() { return m_titleStart; }
    void setTitleStart(int titleStart) { m_titleStart = titleStart; };
    int chapterStart() { return m_chapterStart; }
    void setChapterStart(int chapterStart) { m_chapterStart = chapterStart; };
    const int& titles() const { return m_titles; };
    void setTitles(const int& titles) { m_titles = titles; };
    int chapters() const { return m_chapters; };
    void setChapters(int chapters) { m_chapters = chapters; };
    virtual void setProperty(const QString& name, QVariant value);
    bool isVmgm() const { return m_vmgm; };
    void setVmgm(bool vmgm) { m_vmgm = vmgm; };
    bool isFirst() const { return m_first; };
    void setFirst(bool first) { m_first = first; };

  protected:
    QSize resolution(const QString &type);

  private:
    Magick::Image m_background;
    Magick::Image m_sub;
    Magick::Image m_subHighlight;
    Magick::Image m_subSelect;
    Magick::Image m_temp;
    QPtrList<KMFButton>* m_buttons;
    QSize m_resolution;
    QString m_language;
    int m_modifiedLayers;
    int m_titles;
    int m_chapters;
    int m_titleStart;
    int m_chapterStart;
    bool m_vmgm;
    bool m_first;
    bool m_directPlay;
    bool m_directChapterPlay;

    bool paintChildWidgets(QObject* parent);
    void saveImages(QString projectPath);
    bool writeSpumuxXml(QDomDocument& doc);
    bool writeSpumuxXml(QString projectPath);
    bool makeScript(QString projectPath);
    void magickReport(Magick::Exception &e, QString s = QString::null);
    void saveImage(Magick::Image& image, const QString& file);
};

#endif
