//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#ifndef VIDEOOBJECT_H
#define VIDEOOBJECT_H

#include <kmediafactory/plugin.h>
#include "rect.h"
#include "font.h"
#include "qdvdinfo.h"
#include <kurl.h>
#include <kprocess.h>
#include <qdatetime.h>
#include <qptrlist.h>
#include <qdatetime.h>

class QFFMpeg;
class QDVD::Subtitle;

/**
*/
class VideoObject : public KMF::MediaObject
{
    Q_OBJECT
  public:
    VideoObject(QObject* parent);
    virtual ~VideoObject();
    virtual void toXML(QDomElement& element) const;
    virtual void fromXML(const QDomElement& element);
    virtual void writeDvdAuthorXml(QDomElement& element,
                                   QString preferredLanguage);
    virtual bool make();
    virtual int timeEstimate() const;
    virtual QPixmap pixmap() const;
    virtual QImage preview(int chapter = MainPreview) const;
    virtual QString text(int chapter = MainTitle) const;
    virtual int chapters() const;

    const QDVD::ChapterList& chapterList() const { return m_chapters; };
    void setChapterList(const QDVD::ChapterList& list) { m_chapters = list; };
    void addChapter(const QDVD::Chapter& chapter)
      { m_chapters.append(chapter); };

    const QDVD::AudioList& audioTracks() const { return m_audioTracks; };
    void setAudioTracks(const QDVD::AudioList& streams)
        { m_audioTracks = streams; };
    void addAudioTrack(const QDVD::AudioTrack& audio)
        { m_audioTracks.append(audio); };

    const QDVD::SubtitleList& subtitles() const { return m_subtitles; };
    void setSubtitles(const QDVD::SubtitleList& subtitles)
        { m_subtitles = subtitles; };
    void addSubtitle(const QDVD::Subtitle& subtitle)
        { m_subtitles.append(subtitle); };

    virtual QDVD::VideoTrack::AspectRatio aspect() const { return m_aspect; };
    void setAspect(QDVD::VideoTrack::AspectRatio aspect) { m_aspect = aspect; };

    virtual void actions(QPtrList<KAction>& actionList) const;
    virtual bool addFile(QString fileName);
    void setTitleFromFileName();
    const KURL& previewURL() const { return m_previewURL; };
    void setPreviewURL(const KURL& previewURL) { m_previewURL = previewURL; };
    QImage getFrame(double time) const;
    const QString& id() const { return m_id; };

    double frameRate() const;
    QTime duration() const;
    virtual QStringList files() const;
    QString fileName() const;

  public slots:
    virtual void slotProperties();
    virtual void slotProgress(int);

  private slots:
    void output(KProcess* proc, char* buffer, int buflen);

  protected:
    mutable QDVD::ChapterList m_chapters;
    QDVD::AudioList m_audioTracks;
    QDVD::SubtitleList m_subtitles;

  private:
    mutable QFFMpeg* m_decoder;
    KAction* m_videoProperties;
    KURL m_previewURL;
    QDVD::VideoTrack::AspectRatio m_aspect;
    QString m_id;
    QString m_buffer;
    QTime m_lastUpdate;
    bool m_stopped;

    void generateId();
    void setChapterSecs(double secs);
    bool isBlack(const QImage& img) const;
    bool convertToDVD(QString& fileName);
    bool convertSubtitles(QString& fileName, const QDVD::Subtitle& subtitle);
    bool writeSpumuxXml(QDomDocument& doc, const QDVD::Subtitle& subtitle);
    bool writeSpumuxXml(const QString& fileName,
                        const QDVD::Subtitle& subtitle);
    QString checkFontFile(const QString& file);
};

#endif // VIDEOOBJECT_H
